/* 
In meinem heutigen SQL-Kurs fragte mich ein Teilnehmer,
wie man Daten aus einer CSV- oder XLS-Datei in den 
Microsoft SQL-Server laden kann.

Dazu gibt es ein ganzes Bndel an Mglichkeiten. Wenn man 
selber Administrator ist und fr den eigenen Bedarf Daten
importieren will, dann wrde ich den Einsatz von OpenRowSet
bevorzugen.

Das erklre ich im folgenden im Rahmen einer Work-Bench:
Man kann den kompletten Code in das Management-Studio laden
und schrittweise ausfhren. Dazu markiert man die Statements, 
die zwischen zwei "go" stehen und drckt auf ausfhren. 
Dadurch werden nur die markierten Befehle ausgefhrt.
*/


-- Schritt 1: OpenRowSet erlauben
-- Zuerst den Zugriff auf externe Quellen mittels OpenRowSet erlauben
-- Achtung: bitte nur fr den internen Bedarf oder privat einsetzen. 

exec sp_configure 'show advanced options', 1
reconfigure with override

exec sp_configure 'Ad Hoc Distributed Queries', 1
reconfigure with override
go
/**************** XLS ********************/

-- Beispiel-Zugriff auf ein Blatt in einer XLS-Datei.
-- Die XLS-Datei ist auf dem Server unter "I:\Openrowset\testing.xls" gespeichert:
select * from OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')

-- Htte das Blatt einen anstndigen Namen, dann wrde man anstelle von "Tabelle1$"
-- diesen Namen schreiben.

go

-- Man kann auch in ein bereits existierendes Excel-Document schreiben:
insert into OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
             'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')
(Nachname, Vorname, Geburtstag, Wohnort)
values ('Bond', 'James', '4.4.1930', 'London') 

-- Kontrolle, ob es geklappt hat:
select * from OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')

go

-- Lschen geht nicht
delete from OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
            'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')
where nachname = 'Bond'

/* Es kommt ein Fehler, weil der Provider das Lschen nicht untersttzt:
OLE DB provider "Microsoft.Jet.OLEDB.4.0" for linked server "(null)" returned message "ISAM untersttzt das Lschen von Daten in einer verknpften Tabelle nicht.".
Msg 7345, Level 16, State 1, Line 1
The OLE DB provider "Microsoft.Jet.OLEDB.4.0" for linked server "(null)" could not delete from table "SELECT * FROM [Tabelle1$]". There was a recoverable, provider-specific error, such as an RPC failure.
*/

go

-- Update geht aber:
Update OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
            'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')
set nachname = 'Blond'
where nachname = 'Bond'

-- Kontrolle, ob es geklappt hat:
select * from OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
'Excel 8.0;Database=I:\Openrowset\testing.xls','SELECT * FROM [Tabelle1$]')

go
/**************** CSV ********************/

-- CSV geht auch mit dem Jet-Treiber
SELECT * FROM OPENROWSET( 'Microsoft.Jet.OLEDB.4.0',
'Text;Database=I:\Openrowset\','SELECT * FROM [testing.csv]')

-- Hier gibt man als "Database" nur das Verzeichnis an, die Datei folgt im Select.

go

-- und auch ber ODBC:
SELECT *
FROM OPENROWSET('MSDASQL',
        'Driver={Microsoft Text Driver (*.txt; *.csv)};DEFAULTDIR=I:\Openrowset\;Extensions=CSV;',
        'SELECT * FROM [testing.csv]')

go
/* "komplizierte" Formate gehen ber ODBC aber nur mit schema.ini

Als Beispiel habe ich die Datei aus http://www.brueder-unitaet.de/download/Losung_2007_CSV.zip
verwendet.
*/

SELECT *
FROM OPENROWSET('MSDASQL',
        'Driver={Microsoft Text Driver (*.txt; *.csv)};
          DEFAULTDIR=I:\Openrowset\;Extensions=CSV;',
        'SELECT * FROM [Losungen Free 2007.csv]')
/* Schema.ini in "I:\Openrowset\":
[losungen free 2006.csv]
ColNameHeader=True
MaxScanRows=0
CharacterSet=ANSI
Format=Delimited(;)
*/


go
-- letzter Schritt: OpenRowSet wieder abschalten

exec sp_configure 'Ad Hoc Distributed Queries', 0
reconfigure with override

exec sp_configure 'show advanced options', 0
reconfigure with override